//*****************************************************************************
//
// usb_dev_bulk.c - Main routines for the generic bulk device example.
//
// Copyright (c) 2013-2017 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 2.1.4.178 of the DK-TM4C129X Firmware Package.
//
//*****************************************************************************

#include <stdbool.h>
#include <stdint.h>
#include "inc/hw_ints.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "grlib/grlib.h"
#include "usblib/usblib.h"
#include "usblib/usb-ids.h"
#include "usblib/device/usbdevice.h"
#include "usblib/device/usbdbulk.h"
#include "utils/uartstdio.h"
#include "utils/ustdlib.h"
#include "drivers/frame.h"
#include "drivers/kentec320x240x16_ssd2119.h"
#include "drivers/pinout.h"
#include "usb_bulk_structs.h"
#include "inc/hw_memmap.h"
#include "driverlib/gpio.h"         //USB-HS, added for USB HS support
#include "driverlib/usb.h"

//#include "driverlib/usb.h"          //USB-HS, needed?

#define ALLOW_MULTIPLE_BULK_TRANSACTIONS_PER_FRAME          //??? does not help the speed

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>USB Generic Bulk Device (usb_dev_bulk)</h1>
//!
//! This example provides a generic USB device offering simple bulk data
//! transfer to and from the host.  The device uses a vendor-specific class ID
//! and supports a single bulk IN endpoint and a single bulk OUT endpoint.
//! Data received from the host is assumed to be ASCII text and it is
//! echoed back with the case of all alphabetic characters swapped.
//!
//! A Windows INF file for the device is provided on the installation media and
//! in the C:/ti/TivaWare-C-Series-X.X/windows_drivers directory of TivaWare
//! releases.  This INF contains information required to install the WinUSB
//! subsystem on WindowsXP and Vista PCs.  WinUSB is a Windows subsystem
//! allowing user mode applications to access the USB device without the need
//! for a vendor-specific kernel mode driver.
//!
//! A sample Windows command-line application, usb_bulk_example, illustrating
//! how to connect to and communicate with the bulk device is also provided.
//! The application binary is installed as part of the ``Windows-side examples
//! for USB kits'' package (SW-USB-win) on the installation CD or via download
//! from http://www.ti.com/tivaware .  Project files are included to allow
//! the examples to be built using Microsoft Visual Studio 2008.  Source code
//! for this application can be found in directory
//! ti/TivaWare-for-C-Series/tools/usb_bulk_example.
//
//*****************************************************************************

#define DEBUG               //Uart is 115200 baud

//*****************************************************************************
//
// The system tick rate expressed both as ticks per second and a millisecond
// period.
//
//*****************************************************************************
#define SYSTICKS_PER_SECOND 100
#define SYSTICK_PERIOD_MS   (1000 / SYSTICKS_PER_SECOND)

//*****************************************************************************
//
// The global system tick counter.
//
//*****************************************************************************
volatile uint32_t g_ui32SysTickCount = 0;

uint16_t g_requestData = 0;
void * g_pData = 0;

//*****************************************************************************
//
// Variables tracking transmit and receive counts.
//
//*****************************************************************************
volatile uint32_t g_ui32TxCount = 0;
volatile uint32_t g_ui32RxCount = 0;
#ifdef DEBUG
uint32_t g_ui32UARTRxErrors = 0;
#endif

//*****************************************************************************
//
// Debug-related definitions and declarations.
//
// Debug output is available via UART0 if DEBUG is defined during build.
//
//*****************************************************************************
#ifdef DEBUG
//*****************************************************************************
//
// Map all debug print calls to UARTprintf in debug builds.
//
//*****************************************************************************
#define DEBUG_PRINT UARTprintf

#else

//*****************************************************************************
//
// Compile out all debug print calls in release builds.
//
//*****************************************************************************
#define DEBUG_PRINT while(0) ((int32_t (*)(char *, ...))0)
#endif

//*****************************************************************************
//
// Graphics context used to show text on the color LCD display.
//
//*****************************************************************************
tContext g_sContext;
#define TEXT_FONT               g_psFontCmss18b

//*****************************************************************************
//
// Flags used to pass commands from interrupt context to the main loop.
//
//*****************************************************************************
#define COMMAND_PACKET_RECEIVED 0x00000001
#define COMMAND_STATUS_UPDATE   0x00000002

volatile uint32_t g_ui32Flags = 0;
char *g_pcStatus;

//*****************************************************************************
//
// Global flag indicating that a USB configuration has been set.
//
//*****************************************************************************
static volatile bool g_bUSBConfigured = false;

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, uint32_t ui32Line)
{
    UARTprintf("Error at line %d of %s\n", ui32Line, pcFilename);
    while(1)
    {
    }
}
#endif


//*****************************************************************************
//
// Interrupt handler for the system tick counter.
//
//*****************************************************************************
void
SysTickIntHandler(void)
{
    //
    // Update our system tick counter.
    //
    g_ui32SysTickCount++;
}

//*****************************************************************************
//
// Receive new data and echo it back to the host.
//
// \param psDevice points to the instance data for the device whose data is to
// be processed.
// \param pi8Data points to the newly received data in the USB receive buffer.
// \param ui32NumBytes is the number of bytes of data available to be processed.
//
// This function is called whenever we receive a notification that data is
// available from the host. We read the data, byte-by-byte and swap the case
// of any alphabetical characters found then write it back out to be
// transmitted back to the host.
//
// \return Returns the number of bytes of data processed.
//
//*****************************************************************************
static uint32_t
EchoNewDataToHost(tUSBDBulkDevice *psDevice, uint8_t *pi8Data,
                  uint_fast32_t ui32NumBytes)
{
    uint_fast32_t ui32Loop, ui32Space, ui32Count;
    uint_fast32_t ui32ReadIndex;
    uint_fast32_t ui32WriteIndex;
    tUSBRingBufObject sTxRing;

    //
    // Get the current buffer information to allow us to write directly to
    // the transmit buffer (we already have enough information from the
    // parameters to access the receive buffer directly).
    //
    USBBufferInfoGet(&g_sTxBuffer, &sTxRing);

    //
    // How much space is there in the transmit buffer?
    //
    ui32Space = USBBufferSpaceAvailable(&g_sTxBuffer);

    //
    // How many characters can we process this time round?
    //
    ui32Loop = (ui32Space < ui32NumBytes) ? ui32Space : ui32NumBytes;
    ui32Count = ui32Loop;

    //
    // Update our receive counter.
    //
    g_ui32RxCount += ui32NumBytes;

    //
    // Dump a debug message.
    //
    DEBUG_PRINT("Received %d bytes\n", ui32NumBytes);

    //return(ui32Count);      //phil, skip echo

    //
    // Set up to process the characters by directly accessing the USB buffers.
    //
    ui32ReadIndex = (uint32_t)(pi8Data - g_pui8USBRxBuffer);
    ui32WriteIndex = sTxRing.ui32WriteIndex;

    while(ui32Loop)
    {
        //
        // Copy from the receive buffer to the transmit buffer converting
        // character case on the way.
        //

        //
        // Is this a lower case character?
        //
        if((g_pui8USBRxBuffer[ui32ReadIndex] >= 'a') &&
           (g_pui8USBRxBuffer[ui32ReadIndex] <= 'z'))
        {
            //
            // Convert to upper case and write to the transmit buffer.
            //
            g_pui8USBTxBuffer[ui32WriteIndex] =
                (g_pui8USBRxBuffer[ui32ReadIndex] - 'a') + 'A';
        }
        else
        {
            //
            // Is this an upper case character?
            //
            if((g_pui8USBRxBuffer[ui32ReadIndex] >= 'A') &&
               (g_pui8USBRxBuffer[ui32ReadIndex] <= 'Z'))
            {
                //
                // Convert to lower case and write to the transmit buffer.
                //
                g_pui8USBTxBuffer[ui32WriteIndex] =
                    (g_pui8USBRxBuffer[ui32ReadIndex] - 'Z') + 'z';
            }
            else
            {
                //
                // Copy the received character to the transmit buffer.
                //
                g_pui8USBTxBuffer[ui32WriteIndex] =
                    g_pui8USBRxBuffer[ui32ReadIndex];
            }
        }

        //
        // Move to the next character taking care to adjust the pointer for
        // the buffer wrap if necessary.
        //
        ui32WriteIndex++;
        ui32WriteIndex =
            (ui32WriteIndex == BULK_BUFFER_SIZE) ? 0 : ui32WriteIndex;

        ui32ReadIndex++;
        ui32ReadIndex = (ui32ReadIndex == BULK_BUFFER_SIZE) ? 0 : ui32ReadIndex;

        ui32Loop--;
    }

    //
    // We've processed the data in place so now send the processed data
    // back to the host.
    //
    USBBufferDataWritten(&g_sTxBuffer, ui32Count);

    DEBUG_PRINT("Wrote %d bytes\n", ui32Count);

    //
    // We processed as much data as we can directly from the receive buffer so
    // we need to return the number of bytes to allow the lower layer to
    // update its read pointer appropriately.
    //
    return(ui32Count);
}

//*****************************************************************************
//
// Shows the status string on the display.
//
// \param psContext is a pointer to the graphics context representing the
// display.
// \param pi8Status is a pointer to the string to be shown.
//
//*****************************************************************************
void
DisplayStatus(tContext *psContext, char *pcStatus)
{
    tRectangle sRectLine;
    int32_t i32Y;

    //
    // Calculate the Y coordinate of the top left of the character cell
    // for our line of text.
    //
    i32Y = (GrContextDpyHeightGet(psContext) / 4) -
           (GrFontHeightGet(TEXT_FONT) / 2);

    //
    // Determine the bounding rectangle for this line of text. We add 4 pixels
    // to the height just to ensure that we clear a couple of pixels above and
    // below the line of text.
    //
    sRectLine.i16XMin = 0;
    sRectLine.i16XMax = GrContextDpyWidthGet(psContext) - 1;
    sRectLine.i16YMin = i32Y;
    sRectLine.i16YMax = i32Y + GrFontHeightGet(TEXT_FONT) + 3;

    //
    // Clear the line with black.
    //
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrRectFill(psContext, &sRectLine);

    //
    // Draw the new status string
    //
    DEBUG_PRINT("%s\n", pcStatus);
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrStringDrawCentered(psContext, pcStatus, -1,
                         GrContextDpyWidthGet(psContext) / 2,
                         GrContextDpyHeightGet(psContext) / 4 , false);
}

//*****************************************************************************
//
// Handles bulk driver notifications related to the transmit channel (data to
// the USB host).
//
// \param pvCBData is the client-supplied callback pointer for this channel.
// \param ulEvent identifies the event we are being notified about.
// \param ulMsgValue is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the bulk driver to notify us of any events
// related to operation of the transmit data channel (the IN channel carrying
// data to the USB host).
//
// \return The return value is event-specific.
//
//*****************************************************************************
uint32_t
TxHandler(void *pvCBData, uint32_t ui32Event, uint32_t ui32MsgValue,
          void *pvMsgData)
{
    //
    // We are not required to do anything in response to any transmit event
    // in this example. All we do is update our transmit counter.
    //
    if(ui32Event == USB_EVENT_TX_COMPLETE)
    {
        g_ui32TxCount += ui32MsgValue;
    }

    //
    // Dump a debug message.
    //
    DEBUG_PRINT("TX complete %d\n", ui32MsgValue);

    return(0);
}

//*****************************************************************************
//
// Handles bulk driver notifications related to the receive channel (data from
// the USB host).
//
// \param pvCBData is the client-supplied callback pointer for this channel.
// \param ui32Event identifies the event we are being notified about.
// \param ui32MsgValue is an event-specific value.
// \param pvMsgData is an event-specific pointer.
//
// This function is called by the bulk driver to notify us of any events
// related to operation of the receive data channel (the OUT channel carrying
// data from the USB host).
//
// \return The return value is event-specific.
//
//*****************************************************************************
uint32_t
RxHandler(void *pvCBData, uint32_t ui32Event, uint32_t ui32MsgValue,
          void *pvMsgData)
{
    int i;
    uint_fast32_t ui32ReadIndex;



    //
    // Which event are we being sent?
    //
    switch(ui32Event)
    {
        //
        // We are connected to a host and communication is now possible.
        //
        case USB_EVENT_CONNECTED:
        {
            g_bUSBConfigured = true;
            g_pcStatus = "Host connected.";
            g_ui32Flags |= COMMAND_STATUS_UPDATE;

            //
            // Flush our buffers.
            //
            USBBufferFlush(&g_sTxBuffer);
            USBBufferFlush(&g_sRxBuffer);

            break;
        }

        //
        // The host has disconnected.
        //
        case USB_EVENT_DISCONNECTED:
        {
            g_bUSBConfigured = false;
            g_pcStatus = "Host disconnected.";
            g_ui32Flags |= COMMAND_STATUS_UPDATE;
            break;
        }

        //
        // A new packet has been received.
        //
        case USB_EVENT_RX_AVAILABLE:
        {
            tUSBDBulkDevice *psDevice;

            //
            // Get a pointer to our instance data from the callback data
            // parameter.
            //
            psDevice = (tUSBDBulkDevice *)pvCBData;

            if(g_pData == 0)
                g_pData = &pvMsgData;


            //
            // Read the new packet and echo it back to the host.
            //
            //return(EchoNewDataToHost(psDevice, pvMsgData, ui32MsgValue));

            //ui32ReadIndex = (uint32_t)((uint8_t *)pvMsgData - g_pui8USBRxBuffer);     //get pointer to current data in USB buffer

            //for(i=0; i < ui32MsgValue; i++, ui32ReadIndex++)
            //    g_ui8image[g_ui32RxCount + i] = g_pui8USBRxBuffer[ui32ReadIndex];

            g_ui32RxCount += ui32MsgValue;      //phil, pretend RX data is read
            //if((g_ui32RxCount % (100 * 64)) == 0)
            //    DEBUG_PRINT("Total Received %d bytes\n", g_ui32RxCount);
            return(ui32MsgValue);


        }

        //
        // Ignore SUSPEND and RESUME for now.
        //
        case USB_EVENT_SUSPEND:
        case USB_EVENT_RESUME:
            break;

        //
        // Ignore all other events and return 0.
        //
        default:
            break;
    }

    return(0);
}

//*****************************************************************************
//
// This is the main application entry function.
//
//*****************************************************************************
int
main(void)
{
    uint_fast32_t ui32TxCount;
    uint_fast32_t ui32RxCount;
    char pcBuffer[16];
    uint32_t ui32SysClock;
    uint32_t ui32PLLRate;

    uint32_t ui32ULPI;                  //USB-HS, added for USB HS support

    //
    // Run from the PLL at 120 MHz.
    //
    ui32SysClock = MAP_SysCtlClockFreqSet((SYSCTL_XTAL_25MHZ |
                                           SYSCTL_OSC_MAIN | SYSCTL_USE_PLL |
                                           SYSCTL_CFG_VCO_480), 120000000);

    //
    // Configure the device pins.
    //
    PinoutSet();

    USBULPIPinoutSet();         //USB-HS, configure and enable ULPI to work with USB High Speed Phy

    //Although it is optional, it is better to assert reset momentarily to get more reliable device recognition
    GPIOPinTypeGPIOOutput(GPIO_PORTG_BASE, GPIO_PIN_1);     //USB-HS, use GPIO PG1 to drive USB3300 reset pin
    GPIOPinWrite(GPIO_PORTG_BASE, GPIO_PIN_1, GPIO_PIN_1);              //USB-HS, assert reset to the USB3300 Phy
    //GPIOPinWrite(GPIO_PORTG_BASE, GPIO_PIN_1, 0);       //USB-HS, release USB3300 reset


    //
    // Initialize the display driver.
    //
    Kentec320x240x16_SSD2119Init(ui32SysClock);

    //
    // Initialize the graphics context.
    //
    GrContextInit(&g_sContext, &g_sKentec320x240x16_SSD2119);

    //
    // Draw the application frame.
    //
    FrameDraw(&g_sContext, "usb-dev-bulk");

#ifdef DEBUG
    //
    // Enable UART0
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);

    //
    // Initialize the UART for console I/O.
    //
    UARTStdioConfig(0, 115200, ui32SysClock);
#endif

    //
    // Not configured initially.
    //
    g_bUSBConfigured = false;

    //
    // Show the various static text elements on the color STN display.
    //
    GrContextFontSet(&g_sContext, TEXT_FONT);
    GrStringDraw(&g_sContext, "Tx bytes:", -1, 110, 100, false);
    GrStringDraw(&g_sContext, "Rx bytes:", -1, 110, 130, false);

    //
    // Enable the system tick.
    //
    ROM_SysTickPeriodSet(ui32SysClock / SYSTICKS_PER_SECOND);
    ROM_SysTickIntEnable();
    ROM_SysTickEnable();



    //
    // Show the application name on the display and UART output.
    //
    DEBUG_PRINT("\nTiva C Series USB bulk device example\n");
    DEBUG_PRINT("---------------------------------\n\n");

    //
    // Tell the user what we are up to.
    //
    DisplayStatus(&g_sContext, " Configuring USB... ");


    /***********************************************/

    //USB-HS, added for USB HS support
    //On the DK-TM4C129X board, there is no Phy Mux, so no need to control the Phy enable pin
    //On the DK-TM4C129X board, the external USB3300 Phy is always on, reset of USB3300 is tied GPIO G1.

    //bUSBBusMode = true;
    //GPIOPinWrite(GPIO_PORTH_BASE, (GPIO_PIN_2 | GPIO_PIN_3), GPIO_PIN_3);
    //
    // Switch to USBHS and Release Reset to the ULPI PHY
    //
    //MAP_GPIOPinWrite(GPIO_PORTD_BASE, GPIO_PIN_7, 0);
    //MAP_GPIOPinWrite(GPIO_PORTE_BASE, GPIO_PIN_5, 0);

    /*****************************************************/


    //
    // Initialize the transmit and receive buffers.
    //
    USBBufferInit(&g_sTxBuffer);
    USBBufferInit(&g_sRxBuffer);


    //
    // Tell the USB library the CPU clock and the PLL frequency.  This is a
    // new requirement for TM4C129 devices.
    //
    SysCtlVCOGet(SYSCTL_XTAL_25MHZ, &ui32PLLRate);
    ui32PLLRate = 0;                                                //USB-HB, must be 0 to use external clock for High Speed PHY
    USBDCDFeatureSet(0, USBLIB_FEATURE_CPUCLK, &ui32SysClock);
    USBDCDFeatureSet(0, USBLIB_FEATURE_USBPLL, &ui32PLLRate);

    //ui32ULPI = USBLIB_FEATURE_ULPI_NONE;
    ui32ULPI = USBLIB_FEATURE_ULPI_HS;                              //USB-HS, set ULPI option for high speed or (FS)full speed
    USBDCDFeatureSet(0, USBLIB_FEATURE_USBULPI, &ui32ULPI);         //this causes USBDBulkInit() to lockup if USB3300 Phy is connected or not




#if(0)          //USB-HS
    //DMA related functions lock up here before USBDBulkInit() is called

    USBEndpointDMAConfigSet(USB0_BASE, USB_EP_1, USB_EP_HOST_IN | USB_EP_DMA_MODE_1 | USB_EP_AUTO_CLEAR);

    // Assign endpoint 2 to DMA channel 3 using Mode 1, no bursting, receive, and enable DMA interrupts.
    USBDMAChannelConfigSet(USB0_BASE, 3, USB_EP_1, USB_DMA_CFG_MODE_1 | USB_DMA_CFG_BURST_NONE | USB_DMA_CFG_DIR_RX | USB_DMA_CFG_INT_EN);
    // Make sure that DMA is not enabled on the endpoint. If DMA is left
    // enabled, the endpoint interrupt does not occur.
    //
    USBEndpointDMADisable(USB0_BASE, 3, USB_EP_HOST_IN);

    //
    // Set the destination address for the transfer to pvBuffer.
    //
    USBDMAChannelAddressSet(USB0_BASE, 3, &g_ui8image);
    //
    // Set the transfer size to 512 bytes and the packet count to 8.
    //
    USBDMAChannelCountSet(USB0_BASE, 3, 512);
    USBEndpointPacketCountSet(USB0_BASE, 3, 512/64);
    //
    // Enable the DMA transfer.
    //
    USBEndpointDMAEnable(USB0_BASE, USB_EP_1, USB_EP_HOST_IN);
    USBDMAChannelEnable(USB0_BASE, 3);
#endif



    GPIOPinWrite(GPIO_PORTG_BASE, GPIO_PIN_1, 0);       //USB-HS, release USB3300 reset

    //
    // Initialize the USB stack for device mode.
    //
    //USBStackModeSet(0, eUSBModeDevice, 0);
    USBStackModeSet(0, eUSBModeForceDevice, 0);         //USB-HS, skip checking VBus and ID pins


    void *pvBulkDevice;

    //
    // Pass our device information to the USB library and place the device
    // on the bus.
    //
    pvBulkDevice = USBDBulkInit(0, &g_sBulkDevice);                    //USB-HS, locks up here, host reports "USB device not recognized"
                                                        //device tree shows unknown USB Device (Device Descriptor Request Failed)



    //
    // Wait for initial configuration to complete.
    //
    DisplayStatus(&g_sContext, "Waiting for host...");


    //code from BP101
#if(1)

    /* Configure EP_0/1 (RAM) IN/OUT start address, 0,64,576 bytes into FIFO
     * Must be on 8 byte boundary EP_0 reserves first 16 Bytes for control port data. */
    //MAP_USBFIFOConfigSet(USB0_BASE, USB_EP_0, 0, USB_FIFO_SZ_64, USB_EP_DEV_OUT);
    MAP_USBFIFOConfigSet(USB0_BASE, USB_EP_1, 64, USB_FIFO_SZ_512, USB_EP_DEV_OUT); //USB_EP_HOST_OUT
    MAP_USBFIFOConfigSet(USB0_BASE, USB_EP_1, 576, USB_FIFO_SZ_512, USB_EP_DEV_IN); //USB_EP_HOST_IN

    /* Configure IN endpoint DMA channel mode 1 typically used for transfers that
     * span multiple packets and do not require interrupts between packets.
     * Configure IN endpoint DMA channel mode 0 typically used for transfers
     * not spanning multiple packets or when INTS are required for each packet. */
    MAP_USBEndpointDMAConfigSet(USB0_BASE, USB_EP_1,
                            USB_EP_DEV_IN | USB_EP_DMA_MODE_1 | USB_EP_AUTO_SET | USB_EP_AUTO_REQUEST);
    /* Configure DMA channel mode 1 for OUT endpoint 1 */
    MAP_USBEndpointDMAConfigSet(USB0_BASE, USB_EP_1,
                            USB_EP_DEV_OUT | USB_EP_DMA_MODE_1 | USB_EP_AUTO_CLEAR);//USB_EP_AUTO_REQUEST (EPn Host)
    /* Configure USB0 IN/OUT endpoint 1 for 16-1023 byte data packet size.
     * USB_EP_2,USB_EP_3,USB_EP_4,USB_EP_5,USB_EP_6,USB_EP_7 */
    MAP_USBDevEndpointConfigSet(USB0_BASE, USB_EP_1 , 512,
                            (USB_EP_MODE_BULK|USB_EP_AUTO_SET|USB_EP_AUTO_CLEAR));//USB_EP_AUTO_REQUEST (EPn Host)

#endif



/***************************************  USB DMA ************************************/
    //below DMA code are from TivaWare Peripheral Driver Library User's Guide
    //section 32.4.5 - Using the integrated USB DMA Controller - Receiving a Multiple Packets

#if(1)          //USB-HS
    //could not get the DMA to transfer received USB data to memory, not sure EP1, USB_EP_HOST_IN and USB_DMA_CFG_DIR_RX are correct.
    //tried DMA chan 3 and 0, no DMA transfer
    //did not use any data at g_ui8image[]

    uint32_t chan = USBDMANumChannels(USB0_BASE);

    //DMA
    USBEndpointDMAConfigSet(USB0_BASE, USB_EP_1, USB_EP_HOST_IN | USB_EP_DMA_MODE_1 | USB_EP_AUTO_CLEAR);
    //USBEndpointDMAConfigSet(USB0_BASE, USB_EP_1, USB_EP_HOST_OUT| USB_EP_DMA_MODE_1 | USB_EP_AUTO_CLEAR);

    // Assign endpoint 2 to DMA channel 3 using Mode 1, no bursting, receive, and enable DMA interrupts.
    USBDMAChannelConfigSet(USB0_BASE, 0, USB_EP_1, USB_DMA_CFG_MODE_1 | USB_DMA_CFG_BURST_NONE | USB_DMA_CFG_DIR_RX | USB_DMA_CFG_INT_EN);
    // Make sure that DMA is not enabled on the endpoint. If DMA is left
    // enabled, the endpoint interrupt does not occur.
    //
    USBEndpointDMADisable(USB0_BASE, 0, USB_EP_HOST_IN);
    //USBEndpointDMADisable(USB0_BASE, 0, USB_EP_HOST_OUT);

    //
    // Set the destination address for the transfer to pvBuffer.
    //
    USBDMAChannelAddressSet(USB0_BASE, 0, &g_ui8image);
    //
    // Set the transfer size to 512 bytes and the packet count to 8.
    //
    USBDMAChannelCountSet(USB0_BASE, 0, 512);
    //USBEndpointPacketCountSet(USB0_BASE, 3, 512/64);
    USBEndpointPacketCountSet(USB0_BASE, 0, 100);               //receive large file
    //
    // Enable the DMA transfer.
    //
    USBEndpointDMAEnable(USB0_BASE, USB_EP_1, USB_EP_HOST_IN);
    //USBEndpointDMAEnable(USB0_BASE, USB_EP_1, USB_EP_HOST_OUT);
    USBDMAChannelEnable(USB0_BASE, 0);

#endif






    //
    // Clear our local byte counters.
    //
    ui32RxCount = 0;
    ui32TxCount = 0;



    //USB-HS, blink LED 5 times to indicate code passed USBBulkInit();
    volatile uint32_t ui32Loop;
    int count = 5;
    //
    // Enable the GPIO port that is used for the on-board LED.
    //
    //SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);

    //
    // Check if the peripheral access is enabled.
    //
    //while(!SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOQ))
    //{
    //}

    //GPIOPinTypeGPIOOutput(GPIO_PORTQ_BASE, GPIO_PIN_7);     //phil
    GPIOPinTypeGPIOOutput(GPIO_PORTQ_BASE, GPIO_PIN_7 | GPIO_PIN_4);     //phil

    //
    // Loop forever.
    //
    while(count > 0)
    {
        //
        // Turn on the LED.
        //
        //GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_4 | GPIO_PIN_7, GPIO_PIN_4 | GPIO_PIN_7);
        GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_4 , 0x0);
        GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_7, GPIO_PIN_7);


        //
        // Delay for a bit.
        //
        for(ui32Loop = 0; ui32Loop < 200000; ui32Loop++)
        {
            ;
        }

        //
        // Turn off the LED.
        //
        //GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_7 | GPIO_PIN_4 , 0x0);
        GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_7 , 0x0);
        GPIOPinWrite(GPIO_PORTQ_BASE, GPIO_PIN_4, GPIO_PIN_4);


        //
        // Delay for a bit.
        //
        for(ui32Loop = 0; ui32Loop < 200000; ui32Loop++)
        {
        }
        count--;
    }

    //GrImageDraw(&g_sContext, g_ui8momc, 0, 0);                //phil


    //
    // Main application loop.
    //
    while(1)
    {

        //
        // Have we been asked to update the status display?
        //
        if(g_ui32Flags & COMMAND_STATUS_UPDATE)
        {
            //
            // Clear the command flag
            //
            g_ui32Flags &= ~COMMAND_STATUS_UPDATE;
            DisplayStatus(&g_sContext, g_pcStatus);
        }

        //
        // Has there been any transmit traffic since we last checked?
        //
        if(ui32TxCount != g_ui32TxCount)
        {
            //
            // Take a snapshot of the latest transmit count.
            //
            ui32TxCount = g_ui32TxCount;

            //
            // Update the display of bytes transmitted by the UART.
            //
            usnprintf(pcBuffer, 16, " %d ", ui32TxCount);
            GrStringDraw(&g_sContext, pcBuffer, -1, 190, 100, true);
        }

        //
        // Has there been any receive traffic since we last checked?
        //
        if(ui32RxCount != g_ui32RxCount)
        {
            //
            // Take a snapshot of the latest receive count.
            //
            ui32RxCount = g_ui32RxCount;

            //
            // Update the display of bytes received by the UART.
            //

            GrImageDraw(&g_sContext, g_ui8image, 0, 0);                //phil, display image data that DMA transfer to g_ui8image[], but it is all zero!
            //GrImageDraw(&g_sContext, g_pui8USBRxBuffer, 0, 0);       //phil, display image data directly for huge USB buffer

            usnprintf(pcBuffer, 16, " %d ", ui32RxCount);
            GrStringDraw(&g_sContext, pcBuffer, -1, 190, 130, true);

            //USBBufferInit(&g_sRxBuffer);                    //phil, must initialize buffer again to reset point of RX buffer back to beginning

            //if(g_pData != 0)
            //    *((volatile uint32_t *)g_pData) = (uint32_t)(g_pui8USBRxBuffer);    //must set the data pointer to beginning of buffer
        }

        if(g_requestData != 0)
        {
            usnprintf(pcBuffer, 32, "Request %X data", g_requestData);
            GrStringDraw(&g_sContext, pcBuffer, -1, 60, 160, true);
        }
        if (g_requestData == 0x100)                         //if correct Control command, reset RX buffer pointer
        {
            USBBufferInit(&g_sRxBuffer);                    //phil, must initialize buffer again

            //if(g_pData != 0)
            //    *((volatile uint32_t *)g_pData) = (uint32_t)(g_pui8USBRxBuffer);    //must set the data pointer to beginning of buffer; not needed, use USBBufferInit()
            g_requestData = 0;
        }



    }
}
